@@ -11,13 +11,14 @@ from astropy.time import TimeDelta
 from glob import glob
 from pyoperators import (
     DenseBlockDiagonalOperator, HomothetyOperator, IdentityOperator,
-    Rotation2dOperator, Rotation3dOperator)
+    ReshapeOperator, Rotation2dOperator, Rotation3dOperator)
 from pyoperators.utils import ifirst
 from pysimulators import (
     Acquisition, CartesianEquatorial2HorizontalOperator,
     CartesianGalactic2EquatorialOperator,
     CartesianHorizontal2EquatorialOperator,
-    CartesianEquatorial2GalacticOperator)
+    CartesianEquatorial2GalacticOperator,
+    ConvolutionTruncatedExponentialOperator)
 from pysimulators.interfaces.healpy import HealpixConvolutionGaussianOperator
 from .calibration import QubicCalibration
 from .instrument import QubicInstrument
@@ -119,13 +120,15 @@ class QubicAcquisition(Acquisition):
         """
         return HealpixConvolutionGaussianOperator(fwhm=fwhm, **keywords)
 
-    def get_detector_response_operator(self, tau=0.01):
+    def get_detector_response_operator(self, tau=None):
         """
         Return the operator for the bolometer responses.
 
         """
+        if tau is None:
+            tau = self.instrument.detector.packed.tau
         sampling_period = self.pointing.get_sampling_period()
-        shapein = (self.get_ndetectors, self.pointing.size)
+        shapein = (self.get_ndetectors(), self.pointing.size)
         return ConvolutionTruncatedExponentialOperator(tau / sampling_period,
                                                        shapein=shapein)
 
@@ -142,6 +145,18 @@ class QubicAcquisition(Acquisition):
         return Rotation3dOperator('X', -4 * self.pointing.angle_hwp,
                                   degrees=True)
 
+    def get_invntt_operator(self):
+        """
+        Return the inverse time-time noise correlation matrix as an Operator.
+
+        """
+
+        l = self.instrument.detector.packed
+        fs = 1 / self.pointing.get_sampling_period()
+        return Acquisition.get_invntt_operator(
+            self, sigma=l.sigma, fknee=l.fknee, fslope=l.fslope,
+            sampling_frequency=fs, ncorr=self.instrument.detector.ncorr)
+
     def get_polarizer_operator(self):
         """
         Return operator for the polarizer grid.
@@ -154,16 +169,19 @@ class QubicAcquisition(Acquisition):
             raise ValueError(
                 'Polarized input not handled by a single detector grid.')
 
+        nd = len(self.instrument)
+        nt = len(self.pointing)
         grid = self.instrument.detector.packed.quadrant // 4
         z = np.zeros(self.get_ndetectors())
         if self.instrument.sky.kind == 'QU':
             data = np.array([0.5 - grid, z]).T[:, None, None, :]
         else:
             data = np.array([z + 0.5, 0.5 - grid, z]).T[:, None, None, :]
-        return DenseBlockDiagonalOperator(data)
+        return ReshapeOperator((nd, nt, 1), (nd, nt)) * \
+            DenseBlockDiagonalOperator(data)
 
     def get_projection_peak_operator(self, rotation=None, dtype=None,
-                                     synthbeam_fraction=None):
+                                     synthbeam_fraction=None, verbose=True):
         """
         Return the projection operator for the peak sampling.
 
@@ -180,7 +198,8 @@ class QubicAcquisition(Acquisition):
         if rotation is None:
             rotation = self.get_rotation_g2i()
         return self.instrument.get_projection_peak_operator(
-            rotation, synthbeam_fraction=synthbeam_fraction, dtype=dtype)
+            rotation, synthbeam_fraction=synthbeam_fraction, dtype=dtype,
+            verbose=verbose)
 
     @classmethod
     def load(cls, filename, instrument=None, selection=None):
